<?php

namespace jrosset\ArrayClasses;

use OutOfRangeException;

/**
 * An array
 */
class ArrayClass extends ImmutableArrayClass {
    /**
     * Set or replace a cell value
     *
     * @param mixed $offset The cell offset
     * @param mixed $value  The cell new value
     *
     * @return $this
     */
    public function set ($offset, $value): self {
        $this->array[$offset] = $value;
        return $this;
    }
    /**
     * Remove a cell
     *
     * @param mixed     $offset                      The cell offset
     * @param bool|null $throwsForNonExistentElement If set, temporarily override {@see ImmutableArrayClass::$throwsForNonExistentElement}
     *
     * @return $this
     */
    public function del ($offset, ?bool $throwsForNonExistentElement = null): self {
        if ($throwsForNonExistentElement ?? $this->throwsForNonExistentElement() && !$this->has($offset)) {
            throw new OutOfRangeException();
        }
        unset($this->array[$offset]);

        return $this;
    }

    /**
     * Create an immutable copy
     *
     * @return ImmutableArrayClass The immutable copy
     */
    public function toImmutableArrayClass (): ImmutableArrayClass {
        return new ImmutableArrayClass($this);
    }

    /**
     * @inheritDoc
     */
    public function offsetSet ($offset, $value): void {
        $this->set($offset, $value);
    }
    /**
     * @inheritDoc
     */
    public function offsetUnset ($offset): void {
        $this->del($offset);
    }
}